package CalculiX;

class EDoubleOperation extends Expression {

  // The operator.
  final Operator op;
  // The operands.
  final Expression left, right;

  EDoubleOperation (Expression left, Operator op, Expression right)
  {
    this.left = left;
    this.op = op;
    this.right = right;
  }

  public Value eval (Environment env) throws EvalException
  {
    // Evaluate the operands.
    Value v1 = left.eval(env);
    Value v2 = right.eval(env);
    // Make sure that they produce double results.
    double d1 = v1.asDouble();
    double d2 = v2.asDouble();
    // Perform the appropriate operation.
    double d;
    switch (op) {
    case ADDITION:
      d = d1 + d2;
      break;
    case SUBTRACTION:
      d = d1 - d2;
      break;
    case MULTIPLICATION:
      d = d1 * d2;
      break;
    case DIVISION:
      d = d1 / d2;
      break;
    case POWER:
      d = Math.pow (d1, d2);
      break;
    default:
      // This cannot happen.
      throw new AssertionError ();
    }
    // Package up the result as a value.
    return new VDouble (d);
    }

}
